-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

local margin = 10;
local labelYOffset = 23;
local checkboxYOffset = 23;
local dropdownYOffset = 30;
local sliderYOffset = 35;
local spacingYOffset = 10;
local buttonYOffset = 30;

local elementIndex, yOffset = {}, {};
local oldValues = {};
local cbShowCrf;

function q:InitOptions()
	q.OPT_SHOWCRF = "OPT_SHOWCRF";
	if QOL_OPTIONS[q.name][q.OPT_SHOWCRF] == nil then
		QOL_OPTIONS[q.name][q.OPT_SHOWCRF] = false;
	end

	q.optionsPanel = CreateFrame("Frame", "QoLOptionsFrame_Core");
	q.optionsPanel.name = "Quality of Life";
	q.optionsPanel.okay = function(self) q:SaveOptions(); end;
	q.optionsPanel.cancel = function(self) q:RevertChanges(); end;
	InterfaceOptions_AddCategory(q.optionsPanel);
	q.optionsPanel:SetScript("OnShow", function()
		q:ShowOptionsPanel();
	end);
	q:ShowOptionsPanel();

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleFrame(q.optionsPanel);
	end
end

function q:ShowOptionsPanel()
	oldValues = q:Deepcopy(QOL_OPTIONS[q.name]);

	if not cbShowCrf then
		cbShowCrf = q:CreateCheckbox(q, q:GetText("Show Blizzard Compact Raid Frames"), function(self)
			QOL_OPTIONS[q.name][q.OPT_SHOWCRF] = self:GetChecked();
			q:SetCrfVisible(QOL_OPTIONS[q.name][q.OPT_SHOWCRF]);
		end);
	end
	cbShowCrf:SetChecked(QOL_OPTIONS[q.name][q.OPT_SHOWCRF]);
end

function q:CreateLabel(addon, text, xOffset, inLine)
	xOffset = xOffset or 0;
	yOffset[addon.name] = yOffset[addon.name] or -margin;
	elementIndex[addon.name] = elementIndex[addon.name] or 0;

	local element = CreateFrame("Frame", "QoLOptionElement_" .. addon.name .. "_" .. elementIndex[addon.name], addon.optionsPanel);
	element:SetPoint("TOPLEFT", margin, yOffset[addon.name]);
	element:SetWidth(1);
	element:SetHeight(23);
	element.fontString = element:CreateFontString("QoLOptionElement_" .. addon.name .. "_" .. elementIndex[addon.name] .. "_FontString", "ARTWORK", "GameFontNormal");
	element.fontString:SetPoint("LEFT", margin + xOffset, 0);
	element.fontString:SetText(text);

	function element.SetText(self, text)
		self.fontString:SetText(text);
	end

	function element.SetXOffset(self, xOffset)
		local point, relativeTo, relativePoint, xOfs, yOfs = self.fontString:GetPoint();
		self.fontString:SetPoint(point, xOffset, yOfs);
	end

	function element.SetTextColor(self, red, green, blue, alpha)
		self.fontString:SetTextColor(red, green, blue, alpha);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleFrame(element);
	end

	if not inLine then
		yOffset[addon.name] = yOffset[addon.name] - labelYOffset;
	end
	elementIndex[addon.name] = elementIndex[addon.name] + 1;
	return element;
end

function q:CreateCheckbox(addon, text, func, xOffset, inLine)
	xOffset = xOffset or 0;
	yOffset[addon.name] = yOffset[addon.name] or -margin;
	elementIndex[addon.name] = elementIndex[addon.name] or 0;

	local element = CreateFrame("CheckButton", "QoLOptionElement_" .. addon.name .. "_" .. elementIndex[addon.name], addon.optionsPanel, "ChatConfigCheckButtonTemplate");
	element:SetPoint("TOPLEFT", margin + xOffset + 5, yOffset[addon.name]);
	element:SetScript("OnClick", func);
	element.fontString = element:CreateFontString("QoLOptionElement_" .. addon.name .. "_" .. elementIndex[addon.name] .. "_FontString", "ARTWORK", "GameFontNormal");
	element.fontString:SetPoint("LEFT", 30, 0);
	element.fontString:SetText(text);

	function element.SetText(self, text)
		self.fontString:SetText(text);
	end

	function element.SetTextColor(self, red, green, blue, alpha)
		self.fontString:SetTextColor(red, green, blue, alpha);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleCheckBox(element);
	end

	if not inLine then
		yOffset[addon.name] = yOffset[addon.name] - checkboxYOffset;
	end
	elementIndex[addon.name] = elementIndex[addon.name] + 1;
	return element;
end

function q:CreateDropdown(addon, initFunc, xOffset)
	xOffset = xOffset or 0;
	yOffset[addon.name] = yOffset[addon.name] or -margin;
	elementIndex[addon.name] = elementIndex[addon.name] or 0;

	local element = CreateFrame("Frame", "QoLOptionElement_" .. addon.name .. "_" .. elementIndex[addon.name], addon.optionsPanel, "UIDropDownMenuTemplate");
	element:SetPoint("TOPLEFT", margin + xOffset - 10, yOffset[addon.name]);
	UIDropDownMenu_SetWidth(element, 160);
	UIDropDownMenu_Initialize(element, initFunc);

	function element.SetText(self, text)
		UIDropDownMenu_SetText(self, text);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleDropDownBox(element);
	end

	yOffset[addon.name] = yOffset[addon.name] - dropdownYOffset;
	elementIndex[addon.name] = elementIndex[addon.name] + 1;
	return element;
end

function q:CreateSlider(addon, min, max, func, xOffset, valueStep, inLine, showMinMax)
	xOffset = xOffset or 0;
	min = min or 0;
	max = max or 100;
	valueStep = valueStep or 1;
	yOffset[addon.name] = yOffset[addon.name] or -margin;
	elementIndex[addon.name] = elementIndex[addon.name] or 0;

	local element = CreateFrame("Slider", "QoLOptionElement_" .. addon.name .. "_" .. elementIndex[addon.name], addon.optionsPanel, "OptionsSliderTemplate");
	element:SetPoint("TOPLEFT", margin + xOffset + 10, yOffset[addon.name]);
	element:SetOrientation("HORIZONTAL");
	element:SetWidth(160);
	element:SetHeight(23);
	element:SetScript("OnValueChanged", func);
	element:SetMinMaxValues(min, max);
	element:SetValueStep(valueStep);
	element:SetObeyStepOnDrag(true);

	if showMinMax then
		getglobal(element:GetName() .. "Low"):SetText(min);
		getglobal(element:GetName() .. "High"):SetText(max);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleSliderFrame(element);
	end

	if not inLine then
		yOffset[addon.name] = yOffset[addon.name] - sliderYOffset;
	end
	elementIndex[addon.name] = elementIndex[addon.name] + 1;
	return element;
end

function q:CreateButton(addon, text, func, xOffset)
	xOffset = xOffset or 0;
	yOffset[addon.name] = yOffset[addon.name] or -margin;
	elementIndex[addon.name] = elementIndex[addon.name] or 0;

	local element = CreateFrame("Button", "QoLOptionElement_" .. addon.name .. "_" .. elementIndex[addon.name], addon.optionsPanel, "UIPanelButtonTemplate");
	element:SetPoint("TOPLEFT", 5 + margin + xOffset, -margin + yOffset[addon.name]);
	element:SetHeight(23);
	element:SetWidth(180);
	element:SetScript("OnClick", func);
	element.fontString = element:CreateFontString("QoLOptionElement_" .. addon.name .. "_" .. elementIndex[addon.name] .. "_FontString", "ARTWORK", "GameFontNormal");
	element.fontString:SetPoint("CENTER");
	element.fontString:SetText(text);

	function element.SetText(self, text)
		self.fontString:SetText(text);
	end

	function element.SetTextColor(self, red, green, blue, alpha)
		self.fontString:SetTextColor(red, green, blue, alpha);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleButton(element);
	end

	yOffset[addon.name] = yOffset[addon.name] - dropdownYOffset;
	elementIndex[addon.name] = elementIndex[addon.name] + 1;
	return element;
end

function q:CreateSpacing(addon)
	yOffset[addon.name] = yOffset[addon.name] or -margin;
	yOffset[addon.name] = yOffset[addon.name] - spacingYOffset;
	return {};
end

function q:RevertChanges()
	QOL_OPTIONS[q.name] = oldValues;
end